#include "X3D_Cylinder.h"

namespace X3DTK {
namespace X3D {

Cylinder::Cylinder()
: X3DGeometry3DNode(), _bottom(true), _radius(1.0f), _height(2.0f), _side(true), _top(true)  
{
  defineTypeName("Cylinder", "Geometry3D");
}

Cylinder::Cylinder(const SFBool &bottom, const SFFloat &radius, const SFFloat &height, const SFBool &side, const SFBool &top)
: X3DGeometry3DNode(), _bottom(bottom), _radius(radius), _height(height), _side(side), _top(top)
{
  defineTypeName("Cylinder", "Geometry3D");
}

Cylinder::Cylinder(const Cylinder &C)
: X3DGeometry3DNode(C), _bottom(C._bottom), _radius(C._radius), _height(C._height), _side(C._side), _top(C._top)
{
}

SFAbstractNode Cylinder::clone() const
{
  return new Cylinder(*this);
}

Cylinder::~Cylinder()
{
}

void Cylinder::setBottom(const SFBool &bottom)
{
  _bottom = bottom;
}

void Cylinder::setRadius(const SFFloat &radius)
{
  _radius = radius;
}

void Cylinder::setHeight(const SFFloat &height)
{
  _height = height;
}

void Cylinder::setSide(const SFBool &side)
{
  _side = side;
}

void Cylinder::setTop(const SFBool &top)
{
  _top = top;
}

void Cylinder::load(const X3DFileElement *element)
{
  int index;
  index = element->getIndexAttribute("bottom");
  if (index != -1)
    _bottom = (element->getAttribute(index).upper() == "TRUE");
  
  index = element->getIndexAttribute("radius");
  if (index != -1)
    _radius = element->getAttribute(index).toFloat(); 

  index = element->getIndexAttribute("height");
  if (index != -1)
    _height = element->getAttribute(index).toFloat(); 
  
  index = element->getIndexAttribute("side");
  if (index != -1)
    _side = (element->getAttribute(index).upper() == "TRUE"); 
  
  index = element->getIndexAttribute("top");
  if (index != -1)
    _top = (element->getAttribute(index).upper() == "TRUE"); 
}

SFString &Cylinder::write(SFString &output) const
{
  if (!_bottom)
    output += " bottom=\"FALSE\"";
  if (_radius != 1.0f)  
    output += " radius=\"" + toSFString(_radius) + "\"";
  if (_height != 2.0f)  
    output += " height=\"" + toSFString(_height) + "\"";
  if (!_side)  
    output += " side=\"FALSE\"";
  if (!_top)    
    output += " top=\"FALSE\"";
      
  return output;
}

}
}
