#include "X3D_CoordinateInterpolator.h"

#include <iostream>

using namespace std;

namespace X3DTK {
namespace X3D {

CoordinateInterpolator::CoordinateInterpolator()
  : X3DInterpolatorNode(),
    _keyValue(MFVec3f())
{
  defineTypeName("CoordinateInterpolator", "Interpolation");
}

CoordinateInterpolator::CoordinateInterpolator(const MFFloat &key,
                                               const MFVec3f &keyValue)
  : X3DInterpolatorNode(key),
    _keyValue(keyValue)
{
  defineTypeName("CoordinateInterpolator", "Interpolation");
}

CoordinateInterpolator::CoordinateInterpolator(const CoordinateInterpolator &N)
  : X3DInterpolatorNode(N),
    _keyValue(N._keyValue)
{
}

SFAbstractNode CoordinateInterpolator::clone() const
{
  return new CoordinateInterpolator(*this);
}

CoordinateInterpolator::~CoordinateInterpolator()
{
}

void CoordinateInterpolator::setKeyValue(const MFVec3f &keyValue)
{
  _keyValue = keyValue;
}

void CoordinateInterpolator::load(const X3DFileElement *element)
{
  X3DInterpolatorNode::load(element);

  int index;
  index = element->getIndexAttribute("keyValue");
  if (index != -1)
    _keyValue = MFVec3f(element->getAttribute(index));
}

SFString &CoordinateInterpolator::write(SFString &output) const
{
  X3DInterpolatorNode::write(output);

  if (!_keyValue.empty())
    output += " keyValue=\"" + toSFString(_keyValue) + "\"";

  return output;  
}

}
}
