#include "X3D_MeshBuilderCoreVisitor.h"
#include "X3D_Scene.h"
#include "MESH_Scene.h"
#include <iostream>

using namespace std;

namespace X3DTK {
namespace X3D {

MeshBuilderCoreVisitor::MeshBuilderCoreVisitor()
: CoreVisitor()
{
  // Enter functions.
  defineEnterFunction(&MeshBuilderCoreVisitor::enterScene);
  defineEnterFunction(&MeshBuilderCoreVisitor::enterX3DNode);
  
  // WalkOn functions.
  defineWalkOnFunction(&MeshBuilderCoreVisitor::walkOnX3DNode);
  
  // Leave functions.
  defineLeaveFunction(&MeshBuilderCoreVisitor::leaveX3DNode);
}

void MeshBuilderCoreVisitor::enterScene(Scene *S) 
{
  MeshBuilderStateVariables *stateVariables = GraphTraversal::getInstanceOf<MeshBuilderStateVariables>();  
  MESH::Scene *MS = dynamic_cast<MESH::Scene *>(stateVariables->getNode(S));
  if (MS == 0)
  {
    MS = new MESH::Scene();
    MS->setName(S->getName());
        
    MS->setFileName(S->getFileName());
    stateVariables->addCoupleNode(S, MS);
  }
  stateVariables->pushNode(MS);
};

void MeshBuilderCoreVisitor::enterX3DNode(X3DNode *) 
{
  MeshBuilderStateVariables *stateVariables = GraphTraversal::getInstanceOf<MeshBuilderStateVariables>();
  stateVariables->pushNode(0);
}

bool MeshBuilderCoreVisitor::walkOnX3DNode(X3DNode *, SFAbstractNode) 
{
  MeshBuilderStateVariables *stateVariables = GraphTraversal::getInstanceOf<MeshBuilderStateVariables>();
  return ((stateVariables->getTop() != 0) && (stateVariables->getChildrenVisit()));
}

void MeshBuilderCoreVisitor::leaveX3DNode(X3DNode *) 
{
  MeshBuilderStateVariables *stateVariables = GraphTraversal::getInstanceOf<MeshBuilderStateVariables>();
  stateVariables->popNode();
};

}
}
